(* Sygnatura Env (srodowisko) i domyslna struktura. *)

signature Env = sig

  (* dowolny eqtyp zmiennej *)
  eqtype var;

  (* dowolny typ wartosci zmiennej *)
  type value;

  (* zmienna niezdefiniowana *)
  exception Undefined of var;

  (* srodowisko (uzywane jako var -> value) *)
  type env;

  (* zmienna := wartosc   *)
  val set: var -> value -> env -> env;

  (* zmienna := wartosc() *)
  val setLazy: var -> (unit -> value) -> env -> env;
    
  (* pobranie wartosci zmiennej *)
  val get: env -> var -> value;

  (* usuniecie zmiennej ze srodowiska *)
  val undef: var -> env -> env;
  
  (* domyslne srodowisko (nie musi byc koniecznie puste) *)
  val empty: env;

  (* czy zmienna jest zdefiniowana *)
  val defined: env -> var -> bool;
  
  (* czy zmienna jest niezdefiniowana *)
  val undefined: env -> var -> bool;
  
  axiom Forall (env,v) => undefined env v = not (defined env v);
  
  (* rownosc dwoch wartosci --- przydatne tylko do aksjomatow *)
  val eq: value * value -> bool;
  
  axiom Forall (env,v) => undefined (undef v env) v;
  
  axiom Forall (env,v) => undefined env v implies (get env v raises Undefined v);

  axiom Forall (env,v,x) => eq(get (set v x env) v, x);

  axiom Forall (env,v,x) => eq(get (setLazy v x env) v, x());
    
  axiom Forall (env,v1,v2,x) =>
    v1<>v2 implies eq(get (set v1 x env) v2, get env v2);

  axiom Forall (env,v1,v2,x) =>
    v1<>v2 implies eq(get (setLazy v1 x env) v2, get env v2);

  end;

(***************************************************************************)

(* Implementacja srodowiska przy uzyciu listy. *)

functor Env(eqtype var; type value) :> Env 
where type var=var and type value=value = struct

  type var = var;
  type value = value;

  fun eq (v1, v2) = ?;

  exception Undefined of var;

  type env = (var*(value Susp.susp)) list;
  
  fun undef key ((v,x) :: t) =
      if key=v then undef key t else (v,x) :: undef key t
    | undef key nil = nil;

  fun setLazy v lx e = (v,Susp.delay lx) :: (undef v e);

  fun set v x e = setLazy v (fn () => x) e

  fun get nil v = raise Undefined v
    | get ((v2,x)::t) v = if v=v2 then Susp.force x else get t v;

  val empty = nil;

  fun defined nil v = false
    | defined ((v,_)::t) v2 = (v=v2) orelse defined t v2;

  fun undefined env v = not (defined env v);

  end;
