(* Ta sygnatura opisuje obrazy (mapy bitowe). Obraz ma ustalona szerokosc (x) *)
(* i wysokosc (y) (w pikselach). Piksele zapisuje sie jako trojki liczb rze-  *)
(* czywistych z przedzialu [0,1]. Obrazek mozna zapisac lub odczytac (w for-  *)
(* macie bmp).                                                                *)

signature Image = sig

    type color = Color.Vector.t;
    
    (* rysunek *)
    type image;
    
    (* nowy rysunek o podanych rozmiarach (x,y) *)
    val newimage: int * int -> image
    
    (* rozmiar rysunku *)
    val xsize: image -> int
    val ysize: image -> int
    
    axiom Forall (x,y) => xsize(newimage(x,y)) = x;
    axiom Forall (x,y) => ysize(newimage(x,y)) = y;
    
    (* pobierz piksel (x,y) *)
    val getpixel: (image * int * int) -> color;
    (* ustaw piksel (x,y) *)
    val putpixel: (image * int * int * color) -> unit;
    
    (* sprawdz, czy dany obrazek i podane wspolrzedne piksela sa poprawne *)
    
    val imageok: image -> int * int -> bool;
    
    axiom Forall (img,x,y,c) =>
      imageok img (x,y) implies
      (putpixel(img,x,y,c); getpixel(img,x,y) = c);

    axiom Forall (img,x1,y1,x2,y2,c) => 
      imageok img (x1,y1) andalso imageok img (x2,y2) andalso
      (x1,y1) <> (x2,y2) implies
      getpixel(img,x2,y2) = (putpixel(img,x1,y1,c); getpixel(img,x2,y2));

    (* zapisz (w formacie bmp) *)
    (* xsize musi byc podzielne przez 4 *)
    (* (ze wzgledu na drobny problem techniczny) *)
    val savebmp: string * image -> unit;

    (* wczytaj (format bmp) *)
    (* xsize musi byc podzielne przez 4 *)
    val loadbmp: string -> image;

    end;
