(* mapy bitowe *)

structure Image :> Image = struct

  open Array;
  open BinIO;

  type color = Color.Vector.t;
  
  (* wysokosc, szerokosc, tablica sx*sy kolorow *)
  type image = (int * int * color array);

  (* tylko czesciowa implementacja, nie sprawdza poprawnosci img *)
  fun imageok ((sx,sy,tab):image) (x,y) = 
    x>=0 andalso x<sx andalso y>=0 andalso y<sy;
  
  fun newimage (sx,sy) = (sx, sy, array(sx*sy, (0.0, 0.0, 0.0)));

  fun xsize ((sx, sy, tab):image) = sx;
  fun ysize ((sx, sy, tab):image) = sy;

  fun getpixel ((sx, sy, tab):image, x, y) =
    Array.sub(tab, sx*(sy-1-y)+x);
 
  fun putpixel ((sx, sy, tab):image, x, y, pix) =
    Array.update(tab, sx*(sy-1-y)+x, pix);

  (* w ten sposob przerabiamy liczby rzeczywiste na/z 0-255 *)
  fun realto8 real = 
    if real<0.0 then 0 else
    if real>1.0 then 255 else
    (Real.round (real * 255.0))
  
  fun realfrom8 w = (Real.fromInt w) / 255.0

  (* format bmp *)
  fun savebmp (s, (sx,sy,tab):image) = let

    val _ = if sx mod 4 = 0 then () else raise Fail "Width must be a multiple of 4"
    
    val os = BinIO.openOut(s)
    
    (* zapisz liczbe 1-, 2- lub 4-bajtowa *)
    fun out1 n = output1(os, Word8.fromInt n)
    fun out2 n = (out1 (n mod 256); out1 (n div 256))
    fun out4 n = (out2 (n mod 65536); out2 (n div 65536))

    (* zapisz piksel *)
    fun outpixel (r,g,b) =
      (out1(realto8 b); out1(realto8 g); out1(realto8 r));

    (* rozmiar calego pliku *)
    val fs = sx*sy*3+54

    (* naglowek formatu BMP *)
    val _ = out1 66 (* B *)
    val _ = out1 77 (* M *)
    val _ = out4 fs (* file size *)
    val _ = out4 0  (* reserved *)
    val _ = out4 54 (* offset *)
    val _ = out4 40 (* info size *)
    val _ = out4 sx (* width *)
    val _ = out4 sy (* height *)
    val _ = out2 1  (* planes *)
    val _ = out2 24 (* bits *)
    val _ = out4 0  (* compression *)
    val _ = out4 0  (* image size (0) *)
    val _ = out4 0  (* -- *)
    val _ = out4 0  (* -- *)
    val _ = out4 0  (* -- *)
    val _ = out4 0  (* -- *)
    
    (* mapa bitowa *)
    val _ = Array.app outpixel tab
    
    (*koniec*)
    val _ = BinIO.closeOut(os)

    in ()
    end;

  fun loadbmp s = let
    val is = BinIO.openIn(s)

    fun take (SOME x) = x | take NONE = raise Fail "Unexpected end of file";
    
    (* wczytaj liczbe 1-, 2- lub 4-bajtowa *)
    fun in1 () = Word8.toInt (take (input1 is))
    fun in2 () = in1() + (256 * in1())
    fun in4 () = in2() + (65536 * in2())
    (* wczytaj piksel *)
    fun inpixel _ = let val (b,g,r) = (
      realfrom8 (in1()),
      realfrom8 (in1()),
      realfrom8 (in1())
      ) in (r,g,b) end;
    
    (* przeskocz n bajtow *)
    fun skip n = (inputN (is, n); ())

    (* czytamy interesujace nas czesci naglowka BMP *)
    val hdr = in2()
    val _ = skip 8
    val ss = in4()
    val _ = in4()
    val sx = in4()
    val sy = in4()
    val _ = if hdr = 19778 then () else raise Fail "Not a 24-bit bitmap"
    val _ = if sx mod 4 = 0 then () else raise Fail "Width must be a multiple of 4"
    val _ = skip (ss - 26)

    (* nowy obrazek ... *)
    val img as (sx,sy,tab) = newimage (sx,sy)
    
    (* czytamy mape bitowa *)
    val _ = Array.modify inpixel tab
    
    (* koniec *)
    val _ = BinIO.closeIn(is)
    in img
    end;
  
  end;
