(* trojwymiarowy obiekt zlozony z pojedynczych pikseli *)

structure Scene :> Scene = struct

  open Image;
  
  fun xfor 0 f = f(0) | xfor x f = (f(x-1); xfor (x-1) f);

  fun xyfor x y f = xfor y (fn yy => xfor x (fn xx => f (xx,yy)));

  open Image;
  
  type spcolor = Color.Vector.t * real;

  type scenepixel =
    int * int * real * spcolor;

  type scene = scenepixel list;

  fun cover c1 (c2,t) : color = Color.Vector.add(Color.Vector.mul(t,c1), c2);
  
  fun sort nil = nil
    | sort ((h as (z1:real,_))::t) = let
      val (l1,l2) = List.partition (fn (z2,_) => z1>z2) t
      in (sort l1) @ (h::(sort l2))
      end

  fun calcpixel pix nil = pix
    | calcpixel pix ((z,h)::t) = calcpixel (cover pix h) t
    
  fun drawscene scn img = let
    val sx  = Image.xsize img
    val sy  = Image.ysize img
    val im  = newimage (sx,sy)
    val arr = Array.array((sx*sy), nil) : (real*spcolor) list Array.array
    val _   = List.app (fn (x,y,z,sp) => let
      val pos = y*sx+x 
      in Array.update(arr, pos, (z,sp) :: (Array.sub(arr,pos)))
      end) scn
    val _  = xyfor sx sy (fn (x,y) => (
      putpixel(im, x, y, calcpixel (getpixel(img,x,y)) (sort
      (Array.sub(arr, x+sx*y))
      ))
      ))
    in im
    end;

  end

