(* Typy parsowane, czyli wyrazenie i instrukcja *)

open Var;

(* operatory infiksowe *)
infix 5 ++ --;
infix 6 ** //;

datatype Assignment = 
    ACst of (Var*Expr)  (*liczone od razu*)
  | AFn  of (Var*Expr)  (*liczone przy uzyciu*)
  | ANul of Var         (*oddefiniuj *)

(* wyrazenia *)
and Expr =
  EV of Var | EC of real  |
  op ++ of (Expr * Expr)  |
  op -- of (Expr * Expr)  |
  op ** of (Expr * Expr)  |
  op // of (Expr * Expr)  |
  EFSin of Expr | EFCos of Expr | EFSqrt of Expr | EFSqr of Expr |
  EFExp of Expr | EFLn of Expr | EFAtan of Expr |
  EFAny of ((real->real)*Expr) |
  EIf of (Expr*Expr*Expr) | ELocal of (Expr * Assignment)
  | ERGB of (Expr * Expr * Expr) | EImage of (Image.image * Expr * Expr)
  | ERed of Expr | EGreen of Expr | EBlue of Expr
  | ENum of Expr;

datatype Rot = RotXY | RotYZ | RotZX;

(* instrukcje *)
datatype Instr =
  INull | ISeq of (Instr*Instr) | IInteractive |
  IQuit | IValue of Expr | IDiff of (Var * Expr) | IInfo | IInclude of string |
  IWhile of (Expr*Instr) | IIf of (Expr*Instr*Instr) |
  ISet of Assignment | 
  ISize of (Expr*Expr) | IDrawBG |
  IClear | ISurf | ICurve | IPoint of (Expr*Expr*Expr) | 
  IMerge | ISave of (string) | IView of (string) |
  IRot0 | IRot of (Rot*Expr) |
  IGeo of ((Expr*Expr) * (Expr*Expr) * Expr) |
  ILoad of string |
  IHelp of string;
